#ifndef __BuddhaGraphicsDevice_H__
#define __BuddhaGraphicsDevice_H__

#include <d3d9.h>
#include "BuddhaGame.h"
#include "BuddhaIndexBuffer.h"
#include "BuddhaModelMeshMaterial.h"
#include "BuddhaTexture2D.h"
#include "BuddhaXNAException.h"

namespace BuddhaXNA
{
	class VertexStream;
	class RenderState;
	class VertexDeclaration;

	/// OtBbNfoCX
	class GraphicsDevice
	{
	protected:
		LPDIRECT3D9 m_pD3D;
		LPDIRECT3DDEVICE9 mD3D9Device;
		VertexStream* m_pVertexStreamCollection[1];

		RenderState* m_pRenderState;

		/// obNobt@̃T[tFCX
		LPDIRECT3DSURFACE9 m_pBackBuffer;

		/// v[e[Vp[^̍T
		D3DPRESENT_PARAMETERS mPresentationParameters;

	public:
		/**
		 * @brief RXgN^
		 * @param deviceType KvȃfoCX^Cv
		 * @param renderWindowHandle tH[JXEBhE
		 * @param creationOptions foCX̑IȐU镑Rg[ЂƂ͂ȏ̃IvV̑gݍ킹
		 * @param presentationParameters 쐬foCX̃v[e[Vp[^B
		 */
		GraphicsDevice(D3DDEVTYPE deviceType, HWND renderWindowHandle, DWORD creationOptions, D3DPRESENT_PARAMETERS &presentationParameters);

		virtual ~GraphicsDevice();

		LPDIRECT3DDEVICE9 _getD3D9Device(void) const
		{
			return mD3D9Device;
		}

		void Clear(D3DCOLOR color);

		void Clear(DWORD options, D3DCOLOR color, float depth, int stencil);

		inline int DrawPrimitive(D3DPRIMITIVETYPE primitiveType, UINT startVertex, UINT primitiveCount)
		{
			return mD3D9Device->DrawPrimitive(primitiveType, startVertex, primitiveCount);
		}

		inline void DrawIndexedPrimitive(D3DPRIMITIVETYPE primitiveType, int baseVertex, int minVertexIndex, int numVertices, int startIndex, int primitiveCount)
		{
			mD3D9Device->DrawIndexedPrimitive(primitiveType, baseVertex, minVertexIndex, numVertices, startIndex, primitiveCount);
		}

		VertexStream* Vertices(int number)
		{
			// ToDo Check
			return m_pVertexStreamCollection[number];
		}

		void SetIndexBuffer(const IndexBuffer* indexBuffer);

		void SetIndexBuffer(const IndexBufferPtr &indexBuffer);

		void SetVertexDeclaration(VertexDeclaration* declaration);

		RenderState* GetRenderState()
		{
			return m_pRenderState;
		}

		RenderState* RenderState()
		{
			return m_pRenderState;
		}

		/// w肵eNX`Xe[W stage ɃeNX` tex Zbg
		void SetTexture(int statge, Texture2D* tex);

		/// ̃foCX̐V_[^[QbgZbg܂
		void SetRenderTarget(int renderTargetIndex, RenderTarget2D* renderTarget);

		/**
		 * @brief _\tgEFAƃn[hEFAŐ؂ւ
		 * @param bSoftware ^̂ƂA\tgEFA_gp
		 * @remarks
		 *     D3DCREATE_MIXED_VERTEXPROCESSING ō쐬ꂽfoCX́AftHgŃn[hEFA
		 *     _gpB
		 * \par
         *     Œ@\_VF[_ݒ肷ɂ́Ã\bh false nǍŒ@\_
		 *     tH[}bg _setFVF() ĂяoB
		 *
		 * @attention
		 *     HardwareEffect płA_ł͊OϋɓIɌĂяoׂł͂Ȃ
		 * @internal
		 */
		void _setSoftwareVertexProcessing(bool bSoftware)
		{
			_getD3D9Device()->SetSoftwareVertexProcessing(bSoftware);

			if (bSoftware == false)
			{
				_getD3D9Device()->SetVertexShader(NULL);
			}
		}

		/**
		 * n[hEFA_錾foCXɃZbg
		 */
		void _setFVF(DWORD fvf)
		{
			_getD3D9Device()->SetFVF(fvf);
		}

		/**
		 * n[hEFA_OɃ}eAZbg
		 */
		void _setMaterial(const ModelMeshMaterial* material)
		{
			_getD3D9Device()->SetMaterial(&material->_D3D9Material);

			if (!material->TexturePtr.IsNull())
				_getD3D9Device()->SetTexture(0, material->TexturePtr->_getD3D9Texture());
			else
				_getD3D9Device()->SetTexture(0, NULL);
		}

		/** 
		 * @brief GraphicsDevice LobNobt@̃V[NlXɂ鎟̃obt@̓eŕ\Ȃ
		 * @exception DeviceLostException OtBbNfoCX͎ꂢ邪A܃ZbgłȂBă_O͕s\łB
		 */
		void Present(void)
		{
			HRESULT hr = _getD3D9Device()->Present(NULL, NULL, NULL, NULL);
			if (hr == D3DERR_DEVICELOST)
				throw DeviceLostException();
		}

		/**
		 * @brief ݂̃v[e[Vp[^păOtBbNfoCXZbg
		 * @exception DeviceLostException OtBbNfoCX͎ꂢ邪A܃ZbgłȂBă_O͕s\łB
		 */
		void Reset(void)
		{
			Reset(mPresentationParameters);
		}

		/**
		 * @brief w肵v[e[Vp[^păOtBbNfoCXZbg
		 * @exception DeviceLostException OtBbNfoCX͎ꂢ邪A܃ZbgłȂBă_O͕s\łB
		 */
		void Reset(D3DPRESENT_PARAMETERS &presentationParameters);

		/// ̃OtBbNXfoCXɊ֘AĂv[e[Vp[^擾
		const D3DPRESENT_PARAMETERS &GetPresentationParameters(void) const
		{
			return mPresentationParameters;
		}
	};

	/// _[Xe[g
	class RenderState
	{
		friend GraphicsDevice;

	private:
		GraphicsDevice* m_pDevice;

		RenderState(GraphicsDevice *device);

	public:
		void SetPointSize(DWORD value)
		{
			m_pDevice->_getD3D9Device()->SetRenderState(D3DRS_POINTSIZE, value);
		}

		/**
		 * @brief Specifies the culling mode.how back-facing triangles are culled, if at all. The default value is CullMode.CounterClockwise. 
		 * @param mode You may specify this value by using CullMode class members.
		 */
		void SetCullMode(DWORD mode)
		{
			m_pDevice->_getD3D9Device()->SetRenderState(D3DRS_CULLMODE, mode);
		}

		void SetAlphaBlendEnable(bool isEnable)
		{
			m_pDevice->_getD3D9Device()->SetRenderState(D3DRS_ALPHABLENDENABLE, isEnable ? 1 : 0);
		}
	};

	/**
	 * @brief v~eBu^Cvw肷邽߂̃NX
	 * @remarks
	 *     ̃NX́A C# X^C enum lw񋟂邽߂ɑ݂ĂAXNA o҂ C++ ł
	 *     l̃X^CŃv~eBu^Cvw肷邱ƂłB
	 */
	class PrimitiveType
	{
	public:
		static const D3DPRIMITIVETYPE PointList = D3DPT_POINTLIST;
		static const D3DPRIMITIVETYPE TriangleList = D3DPT_TRIANGLELIST;
		static const D3DPRIMITIVETYPE LineList = D3DPT_LINELIST;
		static const D3DPRIMITIVETYPE LineStrip = D3DPT_LINESTRIP;
		static const D3DPRIMITIVETYPE TriangleFan = D3DPT_TRIANGLEFAN;
		static const D3DPRIMITIVETYPE TriangleStrip = D3DPT_TRIANGLESTRIP;
	};

	/**
	 * @brief JO[hw肷邽߂̃NX
	 * @remarks
	 *     ̃NX́A C# X^C enum lw񋟂邽߂ɑ݂ĂAXNA o҂ C++ ł
	 *     l̃X^CŃJO[hw肷邱ƂłB
	 */
	class CullMode
	{
	public:
		static const DWORD CullClockwiseFace = D3DCULL_CW;
		static const DWORD CullCounterClockwiseFace = D3DCULL_CCW;
		static const DWORD None = D3DCULL_NONE;
	};
}

#endif