#ifndef __BuddhaHardwareEffectLight_H__
#define __BuddhaHardwareEffectLight_H__

#include "BuddhaXNAPrerequisites.h"
#include "BuddhaColor.h"
#include "BuddhaVector3.h"

#include <d3d9.h>

namespace BuddhaXNA
{
	class HardwareEffectLightType
	{
	public:
		static const D3DLIGHTTYPE	Point = D3DLIGHT_POINT;
		static const D3DLIGHTTYPE	Spot = D3DLIGHT_SPOT;
		static const D3DLIGHTTYPE	Directional = D3DLIGHT_DIRECTIONAL;
	};

	/**
	 * @brief n[hEFAGtFNgŎgp郉Cg̃C^[tFCX
	 * @remarks
	 *     n[hEFAGtFNggpĂvOAAvO}uVF[_
	 *     ۂɁAR[hύXʂȂčςނ悤ɒ`ĂC^[tFCX
	 */
	class IHardwareEffectLight
	{
	public:
		virtual void SetLightType(D3DLIGHTTYPE type) = NULL;

		virtual void SetDiffuseColor(const Color &color) = NULL;

		virtual void SetSpecularColor(const Color &color) = NULL;

		virtual void SetAmbientColor(const Color &color) = NULL;

		virtual void SetPosition(const Vector3 &position) = NULL;

		virtual void SetDirection(const Vector3 &direction) = NULL;

		virtual void SetFalloff(float falloff) = NULL;

		virtual void SetAttenuation0(float attenuation) = NULL;

		virtual void SetAttenuation1(float attenuation) = NULL;

		virtual void SetAttenuation2(float attenuation) = NULL;

		virtual void SetRange(float range) = NULL;

		virtual void SetTheta(float theta) = NULL;

		virtual void SetPhi(float phi) = NULL;

		virtual void SetEnabled(bool bEnabled) = NULL;
	};

	class HardwareEffectPass;

	/// HardwareEffect p̃CgNX
	class HardwareEffectLight : public IHardwareEffectLight
	{
		friend HardwareEffectPass;

	protected:
		D3DLIGHT9 mD3D9Light;
		bool mEnabled;

	public:
		HardwareEffectLight()
			: mEnabled(false)
		{
		}

		~HardwareEffectLight()
		{
		}

		void SetLightType(D3DLIGHTTYPE type)
		{
			mD3D9Light.Type = type;
		}

		void SetDiffuseColor(const Color &color)
		{
			mD3D9Light.Diffuse = Color::CreateD3DXColor(color);
		}

		void SetSpecularColor(const Color &color)
		{
			mD3D9Light.Specular = Color::CreateD3DXColor(color);
		}

		void SetAmbientColor(const Color &color)
		{
			mD3D9Light.Ambient = Color::CreateD3DXColor(color);
		}

		void SetPosition(const Vector3 &position)
		{
			Vector3::CreateD3DVECTOR(position, mD3D9Light.Position);
		}

		void SetDirection(const Vector3 &direction)
		{
			Vector3::CreateD3DVECTOR(direction, mD3D9Light.Direction);
		}

		void SetFalloff(float falloff)
		{
			mD3D9Light.Falloff = falloff;
		}

		void SetAttenuation0(float attenuation)
		{
			mD3D9Light.Attenuation0 = attenuation;
		}

		void SetAttenuation1(float attenuation)
		{
			mD3D9Light.Attenuation1 = attenuation;
		}

		void SetAttenuation2(float attenuation)
		{
			mD3D9Light.Attenuation2 = attenuation;
		}

		void SetRange(float range)
		{
			mD3D9Light.Range = range;
		}

		void SetTheta(float theta)
		{
			mD3D9Light.Theta = theta;
		}

		void SetPhi(float phi)
		{
			mD3D9Light.Phi = phi;
		}

		void SetEnabled(bool bEnabled)
		{
			mEnabled = bEnabled;
		}
	};
}

#endif