#include "BuddhaGame.h"
#include "BuddhaGraphicsDeviceManager.h"
#include "BuddhaKeyboard.h"
#include <list>

namespace BuddhaXNA
{
	GameComponentCollection::GameComponentCollection()
		: mHasMainloopBegan(false)
	{
	}
	//------------------------------------------------------------------------
	GameComponentCollection::~GameComponentCollection()
	{
		for (GameComponentList::iterator itr = mUpdatableList.begin(); itr != mUpdatableList.end(); ++itr)
			delete (*itr);

		for (DrawableComponentList::iterator itr = mDrawableList.begin(); itr != mDrawableList.end(); ++itr)
			delete (*itr);
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::Add(GameComponent* comp)
	{
		if (mHasMainloopBegan)
		{
			comp->Initialize();
		}

		// Xgɒǉ
		for (GameComponentList::iterator itr = mUpdatableList.begin(), end = mUpdatableList.end();
			itr != end; ++itr)
		{
			if (comp->GetUpdateOrder() >= (*itr)->GetUpdateOrder())
			{
				mUpdatableList.insert(itr, comp);
				return;
			}
		}

		mUpdatableList.push_front(comp);
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::Add(DrawableComponent *comp)
	{
		Add(static_cast<GameComponent*>(comp));

		if (mHasMainloopBegan)
		{
			comp->LoadGraphicsContent(true);
		}

		// Xgɒǉ
		for (DrawableComponentList::iterator itr = mDrawableList.begin(), end = mDrawableList.end();
			itr != end; ++itr)
		{
			if (comp->GetDrawOrder() >= (*itr)->GetDrawOrder())
			{
				++itr;
				mDrawableList.insert(itr, comp);
				return;
			}
		}

		mDrawableList.push_front(comp);
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::Remove(GameComponent *comp)
	{
		//TODO Error
		mUpdatableList.remove(comp);
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::Remove(DrawableComponent *comp)
	{
		//TODO Error
		Remove(static_cast<GameComponent*>(comp));
		mDrawableList.remove(comp);
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::_update(GameTime gameTime)
	{
		for (GameComponentList::iterator itr = mUpdatableList.begin(), end = mUpdatableList.end();
			itr != end; ++itr) {

			if ((*itr)->GetEnable())
				(*itr)->Update(gameTime);
		}
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::_draw(GameTime gameTime)
	{
		for (DrawableComponentList::iterator itr = mDrawableList.begin(), end = mDrawableList.end();
			itr != end; ++itr) {

			if ((*itr)->GetEnable())
				(*itr)->Draw(gameTime);
		}
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::_initializeAll(void)
	{
		if (mHasMainloopBegan)
			return;

		for (GameComponentList::iterator itr = mUpdatableList.begin(), end = mUpdatableList.end();
			itr != end; ++itr)
		{
			(*itr)->Initialize();
		}

		_loadGraphicContent(true);

		mHasMainloopBegan = true;
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::_loadGraphicContent(bool loadAllContent)
	{
		for (DrawableComponentList::iterator itr = mDrawableList.begin(), end = mDrawableList.end();
			itr != end; ++itr)
		{
			(*itr)->LoadGraphicsContent(loadAllContent);
		}
	}
	//------------------------------------------------------------------------
	void GameComponentCollection::_unloadGraphicContent(bool unloadAllContent)
	{
		for (DrawableComponentList::iterator itr = mDrawableList.begin(), end = mDrawableList.end();
			itr != end; ++itr)
		{
			(*itr)->UnloadGraphicContent(unloadAllContent);
		}
	}
}
